@@ -439,12 +439,15 @@ int HashTableResize(HashTable *h, int newsize) {
 
     /* Create a new hash table and rehash everything into it */
     h2 = HashTableCreate(newsize, h->options);
-    
+
     for (i = 0; i < h->nbuckets; i++) {
 	HashItem *hi, *next;
 	for (hi = h->bucket[i]; hi; hi = next) {
-	    uint64_t hv = hash64(h2->options & HASH_FUNC_MASK,
-				 (uint8_t *)hi->key, hi->key_len) & h2->mask;
+	    uint64_t hv = h2->options & HASH_INT_KEYS
+		? hash64(h2->options & HASH_FUNC_MASK,
+			 (uint8_t *)&hi->key, hi->key_len) & h2->mask
+		: hash64(h2->options & HASH_FUNC_MASK,
+			 (uint8_t *)hi->key, hi->key_len) & h2->mask;
 	    next = hi->next;
 	    hi->next = h2->bucket[hv];
 	    h2->bucket[hv] = hi;
@@ -456,6 +459,9 @@ int HashTableResize(HashTable *h, int newsize) {
     h->bucket   = h2->bucket;
     h->nbuckets = h2->nbuckets;
     h->mask     = h2->mask;
+
+    if (h2->hi_pool)
+	pool_destroy(h2->hi_pool);
     free(h2);
 
     return 0;
@@ -495,16 +501,24 @@ HashItem *HashTableAdd(HashTable *h, char *key, int key_len, HashData data,
     if (!key_len)
 	key_len = strlen(key);
 
-    hv = hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len)
-	& h->mask;
+    hv = h->options & HASH_INT_KEYS
+	? hash64(h->options & HASH_FUNC_MASK, (uint8_t *)&key, key_len) & h->mask
+	: hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len) & h->mask;
 
     /* Already exists? */
     if (!(h->options & HASH_ALLOW_DUP_KEYS)) {
 	for (hi = h->bucket[hv]; hi; hi = hi->next) {
-	    if (key_len == hi->key_len &&
-		memcmp(key, hi->key, key_len) == 0) {
-		if (new) *new = 0;
-		return hi;
+	    if (h->options & HASH_INT_KEYS) {
+		if ((int)(size_t)hi->key == (int)(size_t)key) {
+		    if (new) *new = 0;
+		    return hi;
+		}
+	    } else {
+		if (key_len == hi->key_len && key[0] == hi->key[0] &&
+		    memcmp(key, hi->key, key_len) == 0) {
+		    if (new) *new = 0;
+		    return hi;
+		}
 	    }
 	}
     }
@@ -554,8 +568,11 @@ int HashTableDel(HashTable *h, HashItem *hi, int deallocate_data) {
     uint64_t hv;
     HashItem *next, *last;
 
-    hv = hash64(h->options & HASH_FUNC_MASK,
-		(uint8_t *)hi->key, hi->key_len) & h->mask;
+    hv = h->options & HASH_INT_KEYS
+	? hash64(h->options & HASH_FUNC_MASK,
+		 (uint8_t *)&hi->key, hi->key_len) & h->mask
+	: hash64(h->options & HASH_FUNC_MASK,
+		 (uint8_t *)hi->key, hi->key_len) & h->mask;
 
     for (last = NULL, next = h->bucket[hv]; next;
 	 last = next, next = next->next) {
@@ -600,16 +617,18 @@ int HashTableRemove(HashTable *h, char *key, int key_len,
     if (!key_len)
 	key_len = strlen(key);
 
-    hv = hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len)
-	& h->mask;
+    hv = h->options & HASH_INT_KEYS
+	? hash64(h->options & HASH_FUNC_MASK, (uint8_t *)&key, key_len) & h->mask
+	: hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len) & h->mask;
 
     last = NULL;
     next = h->bucket[hv];
 
     while (next) {
 	hi = next;
-	if (key_len == hi->key_len &&
-	    memcmp(key, hi->key, key_len) == 0) {
+	if (((h->options & HASH_INT_KEYS)
+	     ? ((int)(size_t)key == (int)(size_t)hi->key)
+	     : (key_len == hi->key_len && memcmp(key, hi->key, key_len) == 0))) {
 	    /* An item to remove, adjust links and destroy */
 	    if (last)
 		last->next = hi->next;
@@ -649,12 +668,21 @@ HashItem *HashTableSearch(HashTable *h, char *key, int key_len) {
     if (!key_len)
 	key_len = strlen(key);
 
-    hv = hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len)
-	& h->mask;
-    for (hi = h->bucket[hv]; hi; hi = hi->next) {
-	if (key_len == hi->key_len &&
-	    memcmp(key, hi->key, key_len) == 0)
-	    return hi;
+    if (h->options & HASH_INT_KEYS) {
+	hv = hash64(h->options & HASH_FUNC_MASK, (uint8_t *)&key, key_len)& h->mask;
+
+	for (hi = h->bucket[hv]; hi; hi = hi->next) {
+	    if ((int)(size_t)key == (int)(size_t)hi->key)
+		return hi;
+	}
+    } else {
+	hv = hash64(h->options & HASH_FUNC_MASK, (uint8_t *)key, key_len) & h->mask;
+
+	for (hi = h->bucket[hv]; hi; hi = hi->next) {
+	    if (key_len == hi->key_len &&
+		memcmp(key, hi->key, key_len) == 0)
+		return hi;
+	}
     }
 
     return NULL;
@@ -662,7 +690,8 @@ HashItem *HashTableSearch(HashTable *h, char *key, int key_len) {
 
 /*
  * Find the next HashItem (starting from 'hi') to also match this key.
- * This is only valid when the HASH_ALLOW_DUP_KEYS is in use.
+ * This is only valid when the HASH_ALLOW_DUP_KEYS is in use and
+ * we're not using HASH_INT_KEYS.
  *
  * Returns
  *    HashItem if found
@@ -684,12 +713,22 @@ HashItem *HashTableNext(HashItem *hi, char *key, int key_len) {
 /*
  * Dumps a textual represenation of the hash table to stdout.
  */
-void HashTableDump(HashTable *h, FILE *fp) {
+void HashTableDump(HashTable *h, FILE *fp, char *prefix) {
     int i;
     for (i = 0; i < h->nbuckets; i++) {
 	HashItem *hi;
 	for (hi = h->bucket[i]; hi; hi = hi->next) {
-	    fprintf(fp, "%.*s\n", hi->key_len, hi->key);
+	    if (h->options & HASH_INT_KEYS) {
+		fprintf(fp, "%s%d => %"PRId64" (0x%"PRIx64")\n",
+			prefix ? prefix : "",
+			(int)(size_t)hi->key,
+			hi->data.i, hi->data.i);
+	    } else {
+		fprintf(fp, "%s%.*s => %"PRId64" (0x%"PRIx64")\n",
+			prefix ? prefix : "",
+			hi->key_len, hi->key,
+			hi->data.i, hi->data.i);
+	    }
 	}
     }
 }
@@ -1137,7 +1176,8 @@ HashFile *HashFileFopen(FILE *fp) {
 	for (i = 0; i < hf->narchives; i++) {
 	    fnamelen = fgetc(fp);
 	    hf->archives[i] = malloc(fnamelen+1);
-	    fread(hf->archives[i], 1, fnamelen, fp);
+	    if (fnamelen != fread(hf->archives[i], 1, fnamelen, fp))
+		return NULL;
 	    hf->archives[i][fnamelen] = 0;
 	    archive_len += fnamelen+1;
 	}
@@ -1153,8 +1193,10 @@ HashFile *HashFileFopen(FILE *fp) {
 	? (HashFileSection *)malloc(hf->nheaders * sizeof(HashFileSection))
 	: NULL;
     for (i = 0; i < hf->nheaders; i++) {
-	fread(&hf->headers[i].pos,  8, 1, hf->hfp);
-	fread(&hf->headers[i].size, 4, 1, hf->hfp);
+	if (1 != fread(&hf->headers[i].pos,  8, 1, hf->hfp))
+	    return NULL;
+	if (1 != fread(&hf->headers[i].size, 4, 1, hf->hfp))
+	    return NULL;
 	hf->headers[i].archive_no = *(char *)&hf->headers[i].pos;
 	*(char *)&hf->headers[i].pos = 0;
 	hf->headers[i].pos  = be_int8(hf->headers[i].pos) + hf->hh.offset;
@@ -1166,8 +1208,10 @@ HashFile *HashFileFopen(FILE *fp) {
 	? (HashFileSection *)malloc(hf->nfooters * sizeof(HashFileSection))
 	: NULL;
     for (i = 0; i < hf->nfooters; i++) {
-	fread(&hf->footers[i].pos,  8, 1, hf->hfp);
-	fread(&hf->footers[i].size, 4, 1, hf->hfp);
+	if (1 != fread(&hf->footers[i].pos,  8, 1, hf->hfp))
+	    return NULL;
+	if (1 != fread(&hf->footers[i].size, 4, 1, hf->hfp))
+	    return NULL;
 	hf->footers[i].archive_no = *(char *)&hf->footers[i].pos;
 	*(char *)&hf->footers[i].pos = 0;
 	hf->footers[i].pos  = be_int8(hf->footers[i].pos) + hf->hh.offset;
@@ -1365,15 +1409,19 @@ int HashFileQuery(HashFile *hf, uint8_t *key, int key_len,
 	uint64_t pos;
 	uint32_t size;
 
-	fread(k, klen, 1, hf->hfp);
-	fread(&headfoot, 1, 1, hf->hfp);
+	if (1 != fread(k, klen, 1, hf->hfp))
+	    return -1;
+	if (1 != fread(&headfoot, 1, 1, hf->hfp))
+	    return -1;
 	item->header = (headfoot >> 4) & 0xf;
 	item->footer = headfoot & 0xf;
-	fread(&pos, 8, 1, hf->hfp);
+	if (1 != fread(&pos, 8, 1, hf->hfp))
+	    return -1;
 	item->archive = *(char *)&pos;
 	*(char *)&pos = 0;
 	pos = be_int8(pos) + hf->hh.offset;
-	fread(&size, 4, 1, hf->hfp);
+	if (1 != fread(&size, 4, 1, hf->hfp))
+	    return -1;
 	size = be_int4(size);
 	if (klen == key_len && 0 == memcmp(key, k, key_len)) {
 	    item->pos = pos;
@@ -1503,7 +1551,8 @@ char *HashFileExtract(HashFile *hf, char *fname, size_t *len) {
 	    return NULL;
 
 	fseeko(hf->afp[head->archive_no], head->pos, SEEK_SET);
-	fread(&data[pos], head->size, 1, hf->afp[head->archive_no]);
+	if (1 != fread(&data[pos], head->size, 1, hf->afp[head->archive_no]))
+	    return NULL;
 	pos += head->size;
     }
 
@@ -1513,7 +1562,8 @@ char *HashFileExtract(HashFile *hf, char *fname, size_t *len) {
 	return NULL;
 
     fseeko(hf->afp[hfi.archive], hfi.pos, SEEK_SET);
-    fread(&data[pos], hfi.size, 1, hf->afp[hfi.archive]);
+    if (1 != fread(&data[pos], hfi.size, 1, hf->afp[hfi.archive]))
+	return NULL;
     pos += hfi.size;
 
     /* Footer */
@@ -1523,7 +1573,8 @@ char *HashFileExtract(HashFile *hf, char *fname, size_t *len) {
 	    return NULL;
 
 	fseeko(hf->afp[foot->archive_no], foot->pos, SEEK_SET);
-	fread(&data[pos], foot->size, 1, hf->afp[foot->archive_no]);
+	if (1 != fread(&data[pos], foot->size, 1, hf->afp[foot->archive_no]))
+	    return NULL;
 	pos += foot->size;
     }
 
