@@ -16,6 +16,7 @@
 #include <ctype.h>
 #include <limits.h>
 #include "io_lib/os.h"
+#include "io_lib/xalloc.h"
 #ifdef TRACE_ARCHIVE
 #  include <sys/socket.h>
 #  include <netinet/in.h>
@@ -36,10 +37,12 @@
 #include "io_lib/open_trace_file.h"
 #include "io_lib/misc.h"
 #include "io_lib/tar_format.h"
-#include "io_lib/compress.h"
 #include "io_lib/hash_table.h"
-#include "io_lib/sff.h"
-#include "io_lib/srf.h"
+#ifndef SAMTOOLS
+#  include "io_lib/compress.h"
+#  include "io_lib/sff.h"
+#  include "io_lib/srf.h"
+#endif
 
 /*
  * Supported compression extensions. See the magics array in compress.c for
@@ -48,7 +51,8 @@
 static char *magics[] = {"", ".bz", ".gz", ".Z", ".z", ".bz2", ".sz"};
 
 /*
- * Tokenises the search path splitting on colons (unix) or semicolons (windows).
+ * Tokenises the search path splitting on colons (unix) or semicolons
+ * (windows).
  * We also  explicitly add a "./" to the end of the search path
  *
  * Returns: A new search path with items separated by nul chars. Two nul
@@ -58,7 +62,7 @@ static char *magics[] = {"", ".bz", ".gz", ".Z", ".z", ".bz2", ".sz"};
  * The returned data has been malloced. It is up to the caller to free this
  * memory.
  */
-static char *tokenise_search_path(char *searchpath) {
+char *tokenise_search_path(char *searchpath) {
     char *newsearch;
     unsigned int i, j;
     size_t len;
@@ -140,7 +144,7 @@ static mFILE *find_file_tar(char *file, char *tarname, size_t offset) {
 	
 	if (fpind) {
 	    while (fgets(path, PATH_MAX+100, fpind)) {
-		if (cp = strchr(path, '\n'))
+		if ((cp = strchr(path, '\n')))
 		    *cp = 0;
 		tmp_off = strtol(path, &cp, 10);
 		while (isspace(*cp))
@@ -247,6 +251,7 @@ static mFILE *find_file_hash(char *file, char *hashfile) {
     return mfcreate(data, size);
 }
 
+#ifndef SAMTOOLS
 /*
  * Extracts a single trace from an SRF file.
  *
@@ -280,6 +285,7 @@ static mFILE *find_file_srf(char *tname, char *srffile) {
     srf_destroy(srf, 1);
     return mf;
 }
+#endif
 
 #ifdef TRACE_ARCHIVE
 /*
@@ -379,7 +385,7 @@ static mFILE *find_file_archive(char *file, char *arcname) {
 
 #ifdef USE_WGET
 /* NB: non-reentrant due to reuse of handle */
-static mFILE *find_file_url(char *file, char *url) {
+mFILE *find_file_url(char *file, char *url) {
     char buf[8192], *cp;
     mFILE *fp;
     int pid;
@@ -415,7 +421,7 @@ static mFILE *find_file_url(char *file, char *url) {
 #endif
 
 #ifdef HAVE_LIBCURL
-static mFILE *find_file_url(char *file, char *url) {
+mFILE *find_file_url(char *file, char *url) {
     char buf[8192], *cp;
     mFILE *mf = NULL, *headers = NULL;
     int maxlen = 8190 - strlen(file);
@@ -455,11 +461,13 @@ static mFILE *find_file_url(char *file, char *url) {
 	goto error;
     if (0 != curl_easy_setopt(handle, CURLOPT_TIMEOUT, 10L))
 	goto error;
-    if (0 != curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, mfwrite))
+    if (0 != curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION,
+			      (curl_write_callback)mfwrite))
 	goto error;
     if (0 != curl_easy_setopt(handle, CURLOPT_WRITEDATA, mf))
 	goto error;
-    if (0 != curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION, mfwrite))
+    if (0 != curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION,
+			      (curl_write_callback)mfwrite))
 	goto error;
     if (0 != curl_easy_setopt(handle, CURLOPT_WRITEHEADER, headers))
 	goto error;
@@ -507,6 +515,14 @@ static mFILE *find_file_url(char *file, char *url) {
 }
 #endif
 
+#if !defined(USE_WGET) && !defined(HAVE_LIBCURL)
+mFILE *find_file_url(char *file, char *url) {
+    return NULL;
+}
+#endif
+
+
+#ifndef SAMTOOLS
 /*
  * Takes an SFF file in 'data' and edits the header to ensure
  * that it has no index listed and only claims to contain a single entry.
@@ -527,14 +543,13 @@ static mFILE *sff_single(char *data, size_t size) {
 static mFILE *sff_hash_query(char *sff, char *entry, FILE *fp) {
     static HashFile *hf = NULL;
     static char sff_copy[1024];
-    static FILE *fp_copy = NULL;
     char *data;
     size_t size;
 
     /* Cache an open HashFile for fast accessing */
     if (strcmp(sff, sff_copy) != 0) {
 	if (hf) {
-	    hf->afp = hf->hfp = NULL; /* will be closed by our parent */
+	    hf->afp = NULL; hf->hfp = NULL; /* will be closed by our parent */
 	    HashFileDestroy(hf);
 	}
 	fseek(fp, -4, SEEK_CUR);
@@ -542,7 +557,6 @@ static mFILE *sff_hash_query(char *sff, char *entry, FILE *fp) {
 	    return NULL;
 
 	strcpy(sff_copy, sff);
-	fp_copy = fp;
     }
 
     data = HashFileExtract(hf, entry, &size);
@@ -592,11 +606,15 @@ static mFILE *sff_sorted_query(char *sff, char *accno, FILE *fp,
     static char sff_copy[1024];
     unsigned char *us;
     uint32_t start, end;
-    uint32_t offset;
+    uint32_t offset = 0;
     char *data = NULL;
     static char chdr[1024];
     static int chdrlen = 0, nflows = 0;
-    char rhdr[1024];
+    union {
+	char     c1[1024];
+	uint16_t c2[512];
+	uint32_t c4[256];
+    } rhdr;
     int rhdrlen;
     int nbases, dlen;
     int bytes_per_flow = 2;
@@ -702,13 +720,13 @@ static mFILE *sff_sorted_query(char *sff, char *accno, FILE *fp,
      * header to get data length. Then read this too.
      */
     fseek(fp, offset, SEEK_SET);
-    if (16 != fread(rhdr, 1, 16, fp))
+    if (16 != fread(rhdr.c1, 1, 16, fp))
 	return NULL;
 
-    rhdrlen   = be_int2(*(uint16_t *)rhdr);
-    nbases   = be_int4(*(uint32_t *)(rhdr+4));
+    rhdrlen  = be_int2(rhdr.c2[0]);
+    nbases   = be_int4(rhdr.c4[1]);
     
-    if (rhdrlen-16 != fread(rhdr+16, 1, rhdrlen-16, fp))
+    if (rhdrlen-16 != fread(rhdr.c1+16, 1, rhdrlen-16, fp))
 	return NULL;
     dlen = (nflows * bytes_per_flow + nbases * 3 + 7) & ~7;
 
@@ -717,7 +735,7 @@ static mFILE *sff_sorted_query(char *sff, char *accno, FILE *fp,
 	return NULL;
 
     memcpy(data, chdr, chdrlen);
-    memcpy(data + chdrlen, rhdr, rhdrlen);
+    memcpy(data + chdrlen, rhdr.c1, rhdrlen);
     if (dlen != fread(data + chdrlen + rhdrlen, 1, dlen, fp)) {
 	xfree(data);
 	return NULL;
@@ -744,8 +762,13 @@ static mFILE *sff_sorted_query(char *sff, char *accno, FILE *fp,
 static mFILE *find_file_sff(char *entry, char *sff) {
     static FILE *fp = NULL;
     static char sff_copy[1024];
-    char chdr[65536], rhdr[65536]; /* generous, but worst case */
-    uint32_t nkey, nflows, chdrlen, rhdrlen, dlen, magic;
+    union {
+	char     c1[65536];
+	uint16_t c2[32768];
+	uint32_t c4[16384];
+	uint64_t c8[8192];
+    } chdr, rhdr; /* generous, but worst case */
+    uint32_t nflows, chdrlen, rhdrlen = 0, dlen = 0, magic;
     uint64_t file_pos;
     static uint64_t index_offset = 0;
     static uint32_t index_length = 0;
@@ -764,7 +787,8 @@ static mFILE *find_file_sff(char *entry, char *sff) {
     if (strcmp(sff, sff_copy) == 0) {
 	if (memcmp(index_format, ".hsh1.00", 8) == 0) {
 	    return sff_hash_query(sff, entry, fp);
-	} else if (memcmp(index_format, ".srt1.00", 8) == 0) {
+	} else if (memcmp(index_format, ".srt1.00", 8) == 0 ||
+		   memcmp(index_format, ".mft1.00", 8) == 0) {
 	    return sff_sorted_query(sff, entry, fp, index_length-8);
 	}
     }
@@ -779,29 +803,31 @@ static mFILE *find_file_sff(char *entry, char *sff) {
     /* Read the common header */
     if (NULL == (fp = fopen(sff, "rb")))
 	return NULL;
-    if (31 != fread(chdr, 1, 31, fp))
+    if (31 != fread(chdr.c1, 1, 31, fp))
 	return NULL;
 
     /* Check magic & vers: TODO */
-    magic = be_int4(*(uint32_t *)chdr);
+    magic = be_int4(chdr.c4[0]);
     if (magic != SFF_MAGIC)
 	return NULL;
-    if (memcmp(chdr+4, SFF_VERSION, 4) != 0)
+    if (memcmp(chdr.c1+4, SFF_VERSION, 4) != 0)
 	return NULL;
 
     /* If we have an index, use it, otherwise search linearly */
-    index_offset = be_int8(*(uint64_t *)(chdr+8));
-    index_length = be_int4(*(uint32_t *)(chdr+16));
+    index_offset = be_int8(chdr.c8[1]);
+    index_length = be_int4(chdr.c4[4]);
     if (index_length != 0) {
 	long orig_pos = ftell(fp);
 	fseek(fp, index_offset, SEEK_SET);
-	fread(index_format, 1, 8, fp);
+	if (8 != fread(index_format, 1, 8, fp))
+	    return NULL;
 
 	if (memcmp(index_format, ".hsh1.00", 8) == 0) {
 	    /* HASH index v1.00 */
 	    return sff_hash_query(sff, entry, fp);
 
-	} else if (memcmp(index_format, ".srt1.00", 8) == 0) {
+	} else if (memcmp(index_format, ".srt1.00", 8) == 0 ||
+		   memcmp(index_format, ".mft1.00", 8) == 0) {
 	    /* 454 sorted v1.00 */
 	    return sff_sorted_query(sff, entry, fp, index_length-8);
 	} else {
@@ -810,13 +836,13 @@ static mFILE *find_file_sff(char *entry, char *sff) {
 	}
     }
 
-    nreads  = be_int4(*(uint32_t *)(chdr+20));
-    chdrlen = be_int2(*(uint16_t *)(chdr+24));
-    nkey    = be_int2(*(uint16_t *)(chdr+26));
-    nflows  = be_int2(*(uint16_t *)(chdr+28));
+    nreads  = be_int4(chdr.c4[5]);
+    chdrlen = be_int2(chdr.c2[12]);
+    //nkey    = be_int2(chdr.c2[13]);
+    nflows  = be_int2(chdr.c2[14]);
 
     /* Read the remainder of the header */
-    if (chdrlen-31 != fread(chdr+31, 1, chdrlen-31, fp))
+    if (chdrlen-31 != fread(chdr.c1+31, 1, chdrlen-31, fp))
 	return NULL;
 
     file_pos = chdrlen;
@@ -833,21 +859,21 @@ static mFILE *find_file_sff(char *entry, char *sff) {
 	}
 
 	/* Read 16 bytes to get name length */
-	if (16 != fread(rhdr, 1, 16, fp))
+	if (16 != fread(rhdr.c1, 1, 16, fp))
 	    return NULL;
-	rhdrlen   = be_int2(*(uint16_t *)rhdr);
-	name_len = be_int2(*(uint16_t *)(rhdr+2));
-	nbases   = be_int4(*(uint32_t *)(rhdr+4));
+	rhdrlen  = be_int2(rhdr.c2[0]);
+	name_len = be_int2(rhdr.c2[1]);
+	nbases   = be_int4(rhdr.c4[1]);
 
 	/* Read the rest of the header */
-	if (rhdrlen-16 != fread(rhdr+16, 1, rhdrlen-16, fp))
+	if (rhdrlen-16 != fread(rhdr.c1+16, 1, rhdrlen-16, fp))
 	    return NULL;
 
 	file_pos += rhdrlen;
 
 	dlen = (nflows * bytes_per_flow + nbases * 3 + 7) & ~7;
 
-	if (name_len == entry_len  && 0 == memcmp(rhdr+16, entry, entry_len))
+	if (name_len == entry_len  && 0 == memcmp(rhdr.c1+16, entry, entry_len))
 	    break;
 
 	/* This is not the read you are looking for... */
@@ -868,8 +894,8 @@ static mFILE *find_file_sff(char *entry, char *sff) {
     if (NULL == (fake_file = (char *)xmalloc(chdrlen + rhdrlen + dlen)))
 	return NULL;
 
-    memcpy(fake_file, chdr, chdrlen);
-    memcpy(fake_file+chdrlen, rhdr, rhdrlen);
+    memcpy(fake_file, chdr.c1, chdrlen);
+    memcpy(fake_file+chdrlen, rhdr.c1, rhdrlen);
     if (dlen != fread(fake_file+chdrlen+rhdrlen, 1, dlen, fp)) {
 	xfree(fake_file);
 	return NULL;
@@ -878,6 +904,7 @@ static mFILE *find_file_sff(char *entry, char *sff) {
     /* Convert to an mFILE and return */
     return sff_single(fake_file, chdrlen+rhdrlen+dlen);
 }
+#endif
 
 /*
  * Searches for file in the directory 'dirname'. If it finds it, it opens
@@ -896,10 +923,46 @@ static mFILE *find_file_dir(char *file, char *dirname) {
 	len--;
 
     /* Special case for "./" or absolute filenames */
-    if (*file == '/' || (len==1 && *dirname == '.'))
+    if (*file == '/' || (len==1 && *dirname == '.')) {
 	sprintf(path, "%s", file);
-    else
-	sprintf(path, "%.*s/%s", (int)len, dirname, file);
+    } else {
+	/* Handle %[0-9]*s expansions, if required */
+	char *path_end = path;
+	*path = 0;
+	while ((cp = strchr(dirname, '%'))) {
+	    char *endp;
+	    long l = strtol(cp+1, &endp, 10);
+	    if (*endp != 's') {
+		strncpy(path_end, dirname, (endp+1)-dirname);
+		path_end += (endp+1)-dirname;
+		dirname = endp+1;
+		continue;
+	    }
+	    
+	    strncpy(path_end, dirname, cp-dirname);
+	    path_end += cp-dirname;
+	    if (l) {
+		strncpy(path_end, file, l);
+		path_end += MIN(strlen(file), l);
+		file     += MIN(strlen(file), l);
+	    } else {
+		strcpy(path_end, file);
+		path_end += strlen(file);
+		file     += strlen(file);
+	    }
+	    len -= (endp+1) - dirname;
+	    dirname = endp+1;
+	}
+	strncpy(path_end, dirname, len);
+	path_end += MIN(strlen(dirname), len);
+	*path_end = 0;
+	if (*file) {
+	    *path_end++ = '/';
+	    strcpy(path_end, file);
+	}
+
+	//fprintf(stderr, "*PATH=\"%s\"\n", path);
+    }
 
     if (is_file(path)) {
 	return mfopen(path, "rb");
@@ -911,7 +974,7 @@ static mFILE *find_file_dir(char *file, char *dirname) {
      * and we attempt to work out what type by reading the first and last
      * bits of the file.
      */
-    if (cp = strrchr(file, '/')) {
+    if ((cp = strrchr(file, '/'))) {
 	strcpy(path2, path); /* path contains / too as it's from file */
 	*strrchr(path2, '/') = 0;
 
@@ -926,7 +989,8 @@ static mFILE *find_file_dir(char *file, char *dirname) {
 	    if (NULL == (fp = fopen(path2, "rb")))
 		return NULL;
 	    memcpy(magic, "\0\0\0\0\0\0", 4);
-	    fread(magic, 1, 4, fp);
+	    if (4 != fread(magic, 1, 4, fp))
+		return NULL;
 
 	    /* .hsh or .sff at start */
 	    if (memcmp(magic, ".hsh", 4) == 0)
@@ -937,7 +1001,8 @@ static mFILE *find_file_dir(char *file, char *dirname) {
 	    /* Or .hsh or Ihsh at the end */
 	    if (NONE == type) {
 		fseek(fp, -16, SEEK_END);
-		fread(magic, 1, 8, fp);
+		if (8 != fread(magic, 1, 8, fp))
+		    return NULL;
 		if (memcmp(magic+4, ".hsh", 4) == 0)
 		    type = HASH;
 		else if (memcmp(magic, "Ihsh", 4) == 0)
@@ -947,7 +1012,8 @@ static mFILE *find_file_dir(char *file, char *dirname) {
 	    /* or ustar 257 bytes in to indicate un-hashed tar */
 	    if (NONE == type) {
 		fseek(fp, 257, SEEK_SET);
-		fread(magic, 1, 5, fp);
+		if (5 != fread(magic, 1, 5, fp))
+		    return NULL;
 		if (memcmp(magic, "ustar", 5) == 0)
 		    type = TAR;
 	    }
@@ -958,10 +1024,13 @@ static mFILE *find_file_dir(char *file, char *dirname) {
 		return find_file_hash(cp+1, path2);
 	    case TAR:
 		return find_file_tar(cp+1, path2, 0);
+#ifndef SAMTOOLS
 	    case SFF:
 		return find_file_sff(cp+1, path2);
 	    case SRF:
 		return find_file_srf(cp+1, path2);
+#endif
+	    default:
 	    case NONE:
 		break;
 	    }
@@ -1056,6 +1125,7 @@ mFILE *open_path_mfile(char *file, char *path, char *relative_to) {
 		    return fp;
 		}
 #endif
+#ifndef SAMTOOLS
 	    } else if (0 == strncmp(ele2, "SFF=", 4)) {
 		if (valid && (fp = find_file_sff(file2, ele2+4))) {
 		    free(newsearch);
@@ -1067,7 +1137,7 @@ mFILE *open_path_mfile(char *file, char *path, char *relative_to) {
 		    free(newsearch);
 		    return fp;
 		}
-
+#endif
 	    } else {
 		if (valid && (fp = find_file_dir(file2, ele2))) {
 		    free(newsearch);
@@ -1084,9 +1154,9 @@ mFILE *open_path_mfile(char *file, char *path, char *relative_to) {
 	char *cp;
 	char relative_path[PATH_MAX+1];
 	strcpy(relative_path, relative_to);
-	if (cp = strrchr(relative_path, '/'))
+	if ((cp = strrchr(relative_path, '/')))
 	    *cp = 0;
-	if (fp = find_file_dir(file, relative_path))
+	if ((fp = find_file_dir(file, relative_path)))
 	    return fp;
     }
 
