@@ -53,7 +53,7 @@ int vflen(char *fmt, va_list ap)
 {
     int len = 0;
     char *cp, c;
-    long l;
+    long long l;
     int i;
     double d; 
 
@@ -67,10 +67,10 @@ int vflen(char *fmt, va_list ap)
      * provides a va_copy macro to work around this problem. The only system
      * I have seen needing this so far was Linux on AMD64.
      */
-#if defined(NEED_VA_COPY)
+#if defined(HAVE_VA_COPY)
     va_list ap_local;
     va_copy(ap_local, ap);
-    #define ap ap_local
+#    define ap ap_local
 #endif
 
     for(cp = fmt; *cp; cp++) {
@@ -83,7 +83,7 @@ int vflen(char *fmt, va_list ap)
 	    signed int arg_size;
 
 	    /* Firstly, strip the modifier flags (+-#0 and [space]) */
-	    for(; c=*++cp;) {
+	    for(; (c=*++cp);) {
 		if ('#' == c)
 		    len+=2; /* Worst case of "0x" */
 		else if ('-' == c || '+' == c || ' ' == c)
@@ -122,6 +122,10 @@ int vflen(char *fmt, va_list ap)
 	    } else if ('l' == *cp) {
 		arg_size = 1; /* long */
 		cp++;
+		if ('l' == *cp) {
+		    arg_size = 2; /* long long */
+		    cp++;
+		}
 	    } else {
 		arg_size = 0; /* int */
 	    }
@@ -147,6 +151,8 @@ int vflen(char *fmt, va_list ap)
 		    l = (long)va_arg(ap, int);
 		else if (arg_size == 1)
 		    l = va_arg(ap, long); 
+		else if (arg_size == 2)
+		    l = va_arg(ap, long long); 
 		else 
 		    l = (long)va_arg(ap, int);
 
