@@ -46,6 +46,7 @@
 #include <io_lib/ztr.h>
 #include <io_lib/srf.h>
 #include <io_lib/hash_table.h>
+#include <io_lib/xalloc.h>
 
 #define CR 13            /* Decimal code of Carriage Return char */
 #define LF 10            /* Decimal code of Line Feed char */
@@ -167,7 +168,7 @@ int read_filter_from_file(FILE *input, read_filter_t *read_filter)
     char *cFile;                  /* Dynamically allocated buffer (entire file) */
     char *cThisPtr;               /* Pointer to current position in cFile */
 
-    char *filter_type;
+    char *filter_type = NULL;
     char *prefix;
     char *read;
 
@@ -180,10 +181,12 @@ int read_filter_from_file(FILE *input, read_filter_t *read_filter)
     if(cFile == NULL )
 	{
 	    fprintf(stderr, "\nInsufficient memory to read file.\n");
-	    return 0;
+	    return -1;
 	}
 
-    fread(cFile, lFileLen, 1, input); /* Read the entire file into cFile */
+    /* Read the entire file into cFile */
+    if (1 != fread(cFile, lFileLen, 1, input))
+	return -1;
 
     lLineCount  = 0L;
     lTotalChars = 0L;
@@ -454,7 +457,7 @@ void dump_read_filter(read_filter_t *read_filter) {
         for (ibucket=0; ibucket<read_filter->reads_hash->nbuckets; ibucket++) {
             HashItem *hi;
             for (hi = read_filter->reads_hash->bucket[ibucket]; hi; hi = hi->next) {
-                printf("\tread[%d] = %s\n", hi->data.i, hi->key);
+                printf("\tread[%"PRId64"] = %s\n", hi->data.i, hi->key);
             }
         }
     }
@@ -584,7 +587,7 @@ static int add_readpair_region(unsigned int rev_cycle, mFILE *mf) {
 
     /* Initialise */
     c.type     = ZTR_TYPE_REGN;
-    c.data     = data;
+    c.data     = (char *)data;
     c.dlength  = 5;
     c.mdata    = mdata;
     c.mdlength = mdlen;
@@ -592,8 +595,8 @@ static int add_readpair_region(unsigned int rev_cycle, mFILE *mf) {
 
     ztr_mwrite_chunk(mf, &c);
 
-    xfree(data);
-    xfree(mdata);
+    free(data);
+    free(mdata);
     
     return 0;
 }
@@ -610,7 +613,7 @@ static int add_readpair_region(unsigned int rev_cycle, mFILE *mf) {
  */
 int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, int filter_mode, read_filter_t *read_filter, int read_mask, int rev_cycle) {
     srf_t *in_srf;
-    int output_trace_header;
+    int output_trace_header = 0;
     char name[1024];
 
     if (0 == strcmp(input, "-")) {
@@ -635,6 +638,9 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 	ztr_chunk_t *chunk;
 
 	switch(type = srf_next_block_type(in_srf)) {
+	    mFILE *mf;
+	    uint32_t trace_hdr_size;
+
 	case SRFB_CONTAINER:
 	    if (0 != srf_read_cont_hdr(in_srf, &in_srf->ch)) {
 		fprintf(stderr, "Error reading container header.\nExiting.\n");
@@ -684,7 +690,7 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 	    mrewind(in_srf->mf);
 
 	    /* create the trace header data */
-	    mFILE *mf = mfcreate(NULL, 0);
+	    mf = mfcreate(NULL, 0);
 
 	    if (NULL != (in_srf->ztr = partial_decode_ztr(in_srf, in_srf->mf, NULL))) {
 		in_srf->mf_pos = mftell(in_srf->mf);
@@ -696,7 +702,7 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 		mfseek(in_srf->mf, sizeof(ztr_header_t), SEEK_CUR);
 
 		int pos = mftell(in_srf->mf);
-		while (chunk = ztr_read_chunk_hdr(in_srf->mf)) {
+		while ((chunk = ztr_read_chunk_hdr(in_srf->mf))) {
 		    char *key = ztr_lookup_mdata_value(in_srf->ztr, chunk, "TYPE");
 		    int flag = 0;
 
@@ -782,16 +788,17 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 	    in_srf->mf_end = mftell(in_srf->mf);
 
 	    /* construct the new trace header */
-            uint32_t trace_hdr_size = mftell(mf);
+            trace_hdr_size = mftell(mf);
 	    char *trace_hdr;
             if (NULL == (trace_hdr = malloc(trace_hdr_size))) {
-                stderr, "Error making trace header.\nExiting.\n";
+                fprintf(stderr, "Error making trace header.\nExiting.\n");
 		exit(1);
             }
             memcpy(trace_hdr, mf->data, trace_hdr_size);
  	    if (out_srf->th.trace_hdr)
   	        free(out_srf->th.trace_hdr);
-	    srf_construct_trace_hdr(&out_srf->th, in_srf->th.id_prefix, trace_hdr, trace_hdr_size);
+	    srf_construct_trace_hdr(&out_srf->th, in_srf->th.id_prefix,
+				    (uc *)trace_hdr, trace_hdr_size);
             output_trace_header = 1;
 
 	    mfdestroy(mf);
@@ -891,7 +898,7 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 		}
 
 		int pos = mftell(in_srf->mf);
-		while (chunk = ztr_read_chunk_hdr(in_srf->mf)) {
+		while ((chunk = ztr_read_chunk_hdr(in_srf->mf))) {
 		    char *key = ztr_lookup_mdata_value(ztr_tmp, chunk, "TYPE");
 		    int flag = 0;
 
@@ -977,7 +984,10 @@ int srf_filter(char *input, srf_t *out_srf, char chunk_mode, char mdata_mode, in
 
                 /* construct the new trace body */
 		srf_trace_body_t new_tb;
-		srf_construct_trace_body(&new_tb, name+strlen(in_srf->th.id_prefix), -1, mf->data, mf->size, old_tb.flags);
+		srf_construct_trace_body(&new_tb,
+					 name+strlen(in_srf->th.id_prefix), -1,
+					 (uc *)mf->data, mf->size,
+					 old_tb.flags);
 
 		if (0 != srf_write_trace_body(out_srf, &new_tb)) {
 		    fprintf(stderr, "Error writing trace body.\nExiting.\n");
