@@ -44,6 +44,7 @@
 #include <io_lib/ztr.h>
 #include <io_lib/srf.h>
 #include <io_lib/hash_table.h>
+#include <io_lib/xalloc.h>
 
 #define LEVEL_READ  (1 << 0)
 #define LEVEL_CHUNK (1 << 1)
@@ -408,7 +409,7 @@ ztr_t *partial_decode_ztr2(srf_t *srf, mFILE *mf, ztr_t *z) {
 
     /* Load chunks */
     pos = mftell(mf);
-    while (chunk = ztr_read_chunk_hdr(mf)) {
+    while ((chunk = ztr_read_chunk_hdr(mf))) {
 	chunk->data = (char *)xmalloc(chunk->dlength);
 	if (chunk->dlength != mfread(chunk->data, 1, chunk->dlength, mf))
 	    break;
